@@ -1,6 +1,5 @@
 import html
 import json
-import string
 import typing
 import warnings
 from urllib.parse import urlparse
@@ -34,8 +33,10 @@ from django.utils.translation import gettext_lazy as _
 
 from allauth import app_settings as allauth_app_settings
 from allauth.account import app_settings, signals
-from allauth.core import context, ratelimit
+from allauth.core import context
+from allauth.core.internal import ratelimit
 from allauth.core.internal.adapter import BaseAdapter
+from allauth.core.internal.cryptokit import generate_user_code
 from allauth.core.internal.httpkit import (
     headed_redirect_response,
     is_headless_request,
@@ -73,6 +74,7 @@ class DefaultAccountAdapter(BaseAdapter):
         "invalid_login": _("Invalid login."),
         "invalid_password_reset": _("The password reset token was invalid."),
         "max_email_addresses": _("You cannot add more than %d email addresses."),
+        "phone_taken": _("A user is already registered with this phone number."),
         "too_many_login_attempts": _(
             "Too many failed login attempts. Try again later."
         ),
@@ -90,6 +92,7 @@ class DefaultAccountAdapter(BaseAdapter):
         ],
         "select_only_one": _("Please select only one."),
         "same_as_current": _("The new value must be different from the current one."),
+        "rate_limited": _("Be patient, you are sending too many requests."),
     }
 
     def stash_verified_email(self, request, email):
@@ -584,6 +587,13 @@ class DefaultAccountAdapter(BaseAdapter):
         # get_host already validates the given host, so no need to check it again
         allowed_hosts = {context.request.get_host()} | set(settings.ALLOWED_HOSTS)
 
+        # Include hosts derived from CSRF_TRUSTED_ORIGINS
+        trusted_hosts = {
+            urlparse(origin).netloc for origin in settings.CSRF_TRUSTED_ORIGINS
+        }
+        allowed_hosts.update(trusted_hosts)
+
+        # Handle wildcard case
         if "*" in allowed_hosts:
             parsed_host = urlparse(url).netloc
             allowed_host = {parsed_host} if parsed_host else None
@@ -673,15 +683,30 @@ class DefaultAccountAdapter(BaseAdapter):
 
     def _delete_login_attempts_cached_email(self, request, **credentials):
         cache_key = self._get_login_attempts_cache_key(request, **credentials)
-        ratelimit.clear(request, action="login_failed", key=cache_key)
+        # Here, we wipe the login failed rate limit, completely. This is safe,
+        # as we only do this on a succesful password reset, which is rate limited
+        # on itself (e.g. sending of email etc.).
+        ratelimit.clear(
+            request,
+            config=app_settings.RATE_LIMITS,
+            action="login_failed",
+            key=cache_key,
+        )
+
+    def _rollback_login_failed_rl_usage(self) -> None:
+        usage = getattr(self, "_login_failed_rl_usage", None)
+        if usage:
+            usage.rollback()
 
     def pre_authenticate(self, request, **credentials):
         cache_key = self._get_login_attempts_cache_key(request, **credentials)
-        if not ratelimit.consume(
+        self._login_failed_rl_usage = ratelimit.consume(
             request,
+            config=app_settings.RATE_LIMITS,
             action="login_failed",
             key=cache_key,
-        ):
+        )
+        if not self._login_failed_rl_usage:
             raise self.validation_error("too_many_login_attempts")
 
     def authenticate(self, request, **credentials):
@@ -694,7 +719,12 @@ class DefaultAccountAdapter(BaseAdapter):
         alt_user = AuthenticationBackend.unstash_authenticated_user()
         user = user or alt_user
         if user:
-            self._delete_login_attempts_cached_email(request, **credentials)
+            # On a succesful login, we cannot just wipe the login failed rate
+            # limit. That consists of 2 parts, a per IP limit, and, a per
+            # key(email) limit. Wiping it completely would allow an attacker to
+            # insert periodic successful logins during a brute force
+            # process. So instead, we are rolling back our consumption.
+            self._rollback_login_failed_rl_usage()
         else:
             self.authentication_failed(request, **credentials)
         return user
@@ -729,7 +759,7 @@ class DefaultAccountAdapter(BaseAdapter):
             ]
         )
 
-    def get_client_ip(self, request):
+    def get_client_ip(self, request) -> str:
         x_forwarded_for = request.META.get("HTTP_X_FORWARDED_FOR")
         if x_forwarded_for:
             ip = x_forwarded_for.split(",")[0]
@@ -817,32 +847,25 @@ class DefaultAccountAdapter(BaseAdapter):
         """
         Generates a new login code.
         """
-        return self._generate_code()
+        return generate_user_code()
 
     def generate_password_reset_code(self) -> str:
         """
         Generates a new password reset code.
         """
-        return self._generate_code(length=8)
+        return generate_user_code(length=8)
 
     def generate_email_verification_code(self) -> str:
         """
         Generates a new email verification code.
         """
-        return self._generate_code()
+        return generate_user_code()
 
     def generate_phone_verification_code(self) -> str:
         """
         Generates a new phone verification code.
         """
-        return self._generate_code()
-
-    def _generate_code(self, length=6):
-        forbidden_chars = "0OI18B2ZAEU"
-        allowed_chars = string.ascii_uppercase + string.digits
-        for ch in forbidden_chars:
-            allowed_chars = allowed_chars.replace(ch, "")
-        return get_random_string(length=length, allowed_chars=allowed_chars)
+        return generate_user_code()
 
     def is_login_by_code_required(self, login) -> bool:
         """
@@ -872,7 +895,7 @@ class DefaultAccountAdapter(BaseAdapter):
 
         return PhoneField(**kwargs)
 
-    def send_unknown_account_sms(self, phone: str, **kwargs):
+    def send_unknown_account_sms(self, phone: str, **kwargs) -> None:
         """
         In case enumeration prevention is enabled, and, a verification code
         is requested for an unlisted phone number, this method is invoked to
@@ -880,6 +903,9 @@ class DefaultAccountAdapter(BaseAdapter):
         """
         pass
 
+    def send_account_already_exists_sms(self, phone: str) -> None:
+        pass
+
     def send_verification_code_sms(self, user, phone: str, code: str, **kwargs):
         """
         Sends a verification code.
@@ -911,7 +937,7 @@ class DefaultAccountAdapter(BaseAdapter):
     def get_user_by_phone(self, phone: str):
         """
         Looks up a user given the specified phone number. Returns ``None`` if no user
-        was phone.
+        was found.
         """
         raise NotImplementedError
 
