@@ -15,12 +15,15 @@ from django.utils.safestring import mark_safe
 from django.utils.translation import gettext, gettext_lazy as _, pgettext
 
 from allauth.account.app_settings import LoginMethod
+from allauth.account.fields import EmailField, PasswordField, SetPasswordField
 from allauth.account.internal import flows
+from allauth.account.internal.flows.manage_email import email_already_exists
+from allauth.account.internal.flows.phone_verification import (
+    phone_already_exists,
+)
 from allauth.account.internal.flows.signup import base_signup_form_class
-from allauth.account.internal.stagekit import LOGIN_SESSION_KEY
-from allauth.account.internal.textkit import compare_code
-from allauth.account.stages import EmailVerificationStage
 from allauth.core import context, ratelimit
+from allauth.core.internal.cryptokit import compare_user_code
 from allauth.core.internal.httpkit import headed_redirect_response
 from allauth.utils import get_username_max_length, set_form_field_order
 
@@ -63,36 +66,6 @@ class PasswordVerificationMixin:
         return cleaned_data
 
 
-class PasswordField(forms.CharField):
-    def __init__(self, *args, **kwargs):
-        render_value = kwargs.pop(
-            "render_value", app_settings.PASSWORD_INPUT_RENDER_VALUE
-        )
-        kwargs["widget"] = forms.PasswordInput(
-            render_value=render_value,
-            attrs={"placeholder": kwargs.get("label")},
-        )
-        autocomplete = kwargs.pop("autocomplete", None)
-        if autocomplete is not None:
-            kwargs["widget"].attrs["autocomplete"] = autocomplete
-        super(PasswordField, self).__init__(*args, **kwargs)
-
-
-class SetPasswordField(PasswordField):
-    def __init__(self, *args, **kwargs):
-        kwargs["autocomplete"] = "new-password"
-        kwargs.setdefault(
-            "help_text", password_validation.password_validators_help_text_html()
-        )
-        super().__init__(*args, **kwargs)
-        self.user = None
-
-    def clean(self, value):
-        value = super().clean(value)
-        value = get_adapter().clean_password(value, user=self.user)
-        return value
-
-
 class LoginForm(forms.Form):
     password = PasswordField(label=_("Password"), autocomplete="current-password")
     remember = forms.BooleanField(label=_("Remember Me"), required=False)
@@ -104,13 +77,7 @@ class LoginForm(forms.Form):
         super().__init__(*args, **kwargs)
         adapter = get_adapter()
         if app_settings.LOGIN_METHODS == {LoginMethod.EMAIL}:
-            login_widget = forms.EmailInput(
-                attrs={
-                    "placeholder": _("Email address"),
-                    "autocomplete": "email",
-                }
-            )
-            login_field = forms.EmailField(label=_("Email"), widget=login_widget)
+            login_field = EmailField()
         elif app_settings.LOGIN_METHODS == {LoginMethod.USERNAME}:
             login_widget = forms.TextInput(
                 attrs={"placeholder": _("Username"), "autocomplete": "username"}
@@ -305,15 +272,7 @@ class BaseSignupForm(base_signup_form_class()):  # type: ignore[misc]
             attrs={"placeholder": _("Username"), "autocomplete": "username"}
         ),
     )
-    email = forms.EmailField(
-        widget=forms.TextInput(
-            attrs={
-                "type": "email",
-                "placeholder": _("Email address"),
-                "autocomplete": "email",
-            }
-        )
-    )
+    email = EmailField()
 
     def __init__(self, *args, **kwargs):
         self._signup_fields = self._get_signup_fields(kwargs)
@@ -328,7 +287,7 @@ class BaseSignupForm(base_signup_form_class()):  # type: ignore[misc]
 
         email2 = self._signup_fields.get("email2")
         if email2:
-            self.fields["email2"] = forms.EmailField(
+            self.fields["email2"] = EmailField(
                 label=_("Email (again)"),
                 required=email2["required"],
                 widget=forms.TextInput(
@@ -410,7 +369,7 @@ class BaseSignupForm(base_signup_form_class()):  # type: ignore[misc]
     def clean_email(self):
         value = self.cleaned_data["email"].lower()
         value = get_adapter().clean_email(value)
-        if value and app_settings.UNIQUE_EMAIL:
+        if value:
             value = self.validate_unique_email(value)
         return value
 
@@ -418,29 +377,39 @@ class BaseSignupForm(base_signup_form_class()):  # type: ignore[misc]
         value = self.cleaned_data["email2"].lower()
         return value
 
-    def validate_unique_email(self, value):
-        adapter = get_adapter()
-        assessment = flows.manage_email.assess_unique_email(value)
-        if assessment is True:
-            # All good.
-            pass
-        elif assessment is False:
-            # Fail right away.
-            raise adapter.validation_error("email_taken")
-        else:
-            assert assessment is None  # nosec
-            self.account_already_exists = True
-        return adapter.validate_unique_email(value)
+    def validate_unique_email(self, value) -> str:
+        email, self.account_already_exists = flows.manage_email.email_already_exists(
+            value
+        )
+        return email
 
     def clean(self):
-        cleaned_data = super(BaseSignupForm, self).clean()
+        cleaned_data = super().clean()
         if "email2" in self._signup_fields:
             email = cleaned_data.get("email")
             email2 = cleaned_data.get("email2")
             if (email and email2) and email != email2:
                 self.add_error("email2", _("You must type the same email each time."))
+
+        if "phone" in self._signup_fields:
+            self._clean_phone()
         return cleaned_data
 
+    def _clean_phone(self):
+        """Intentionally NOT `clean_phone()`:
+        - phone field is optional (depending on ACCOUNT_SIGNUP_FIELDS)
+        - we don't want to have clean_phone() mistakenly called when a project
+          is using a custom signup form with their own `phone` field.
+        """
+        adapter = get_adapter()
+        if phone := self.cleaned_data.get("phone"):
+            user = adapter.get_user_by_phone(phone)
+            if user:
+                if not app_settings.PREVENT_ENUMERATION:
+                    self.add_error("phone", adapter.error_messages["phone_taken"])
+                else:
+                    self.account_already_exists = True
+
     def custom_signup(self, request, user):
         self.signup(request, user)
 
@@ -452,11 +421,12 @@ class BaseSignupForm(base_signup_form_class()):  # type: ignore[misc]
         if self.account_already_exists:
             # Don't create a new account, only send an email informing the user
             # that (s)he already has one...
-            email = self.cleaned_data["email"]
-            resp = flows.signup.prevent_enumeration(request, email)
+            email = self.cleaned_data.get("email")
+            phone = None
+            if "phone" in self._signup_fields:
+                phone = self.cleaned_data.get("phone")
+            resp = flows.signup.prevent_enumeration(request, email=email, phone=phone)
             user = None
-            # Fake a login stage.
-            request.session[LOGIN_SESSION_KEY] = EmailVerificationStage.key
         else:
             user = self.save(request)
             resp = None
@@ -571,13 +541,7 @@ class UserForm(forms.Form):
 
 
 class AddEmailForm(UserForm):
-    email = forms.EmailField(
-        label=_("Email"),
-        required=True,
-        widget=forms.TextInput(
-            attrs={"type": "email", "placeholder": _("Email address")}
-        ),
-    )
+    email = EmailField(required=True)
 
     def clean_email(self):
         from allauth.account import signals
@@ -661,17 +625,7 @@ class SetPasswordForm(PasswordVerificationMixin, UserForm):
 
 
 class ResetPasswordForm(forms.Form):
-    email = forms.EmailField(
-        label=_("Email"),
-        required=True,
-        widget=forms.TextInput(
-            attrs={
-                "type": "email",
-                "placeholder": _("Email address"),
-                "autocomplete": "email",
-            }
-        ),
-    )
+    email = EmailField(required=True)
 
     def clean_email(self):
         email = self.cleaned_data["email"].lower()
@@ -759,14 +713,7 @@ class ReauthenticateForm(forms.Form):
 
 
 class RequestLoginCodeForm(forms.Form):
-    email = forms.EmailField(
-        widget=forms.EmailInput(
-            attrs={
-                "placeholder": _("Email address"),
-                "autocomplete": "email",
-            }
-        )
-    )
+    email = EmailField()
 
     def __init__(self, *args, **kwargs):
         super().__init__(*args, **kwargs)
@@ -808,15 +755,16 @@ class RequestLoginCodeForm(forms.Form):
     def clean_email(self):
         adapter = get_adapter()
         email = self.cleaned_data["email"]
-        users = filter_users_by_email(email, is_active=True, prefer_verified=True)
-        if not app_settings.PREVENT_ENUMERATION:
-            if not users:
-                raise adapter.validation_error("unknown_email")
-        if not ratelimit.consume(
-            context.request, action="request_login_code", key=email.lower()
-        ):
-            raise adapter.validation_error("too_many_login_attempts")
-        self._user = users[0] if users else None
+        if email:
+            users = filter_users_by_email(email, is_active=True, prefer_verified=True)
+            if not app_settings.PREVENT_ENUMERATION:
+                if not users:
+                    raise adapter.validation_error("unknown_email")
+            if not ratelimit.consume(
+                context.request, action="request_login_code", key=email.lower()
+            ):
+                raise adapter.validation_error("too_many_login_attempts")
+            self._user = users[0] if users else None
         return email
 
 
@@ -829,12 +777,12 @@ class BaseConfirmCodeForm(forms.Form):
     )
 
     def __init__(self, *args, **kwargs):
-        self.code = kwargs.pop("code")
+        self.code = kwargs.pop("code", None)
         super().__init__(*args, **kwargs)
 
     def clean_code(self):
         code = self.cleaned_data.get("code")
-        if not compare_code(actual=code, expected=self.code):
+        if not compare_user_code(actual=code, expected=self.code):
             raise get_adapter().validation_error("incorrect_code")
         return code
 
@@ -844,7 +792,17 @@ class ConfirmLoginCodeForm(BaseConfirmCodeForm):
 
 
 class ConfirmEmailVerificationCodeForm(BaseConfirmCodeForm):
-    pass
+    def __init__(self, *args, **kwargs) -> None:
+        self.user = kwargs.pop("user", None)
+        self.email = kwargs.pop("email", None)
+        super().__init__(*args, **kwargs)
+
+    def clean_code(self) -> str:
+        code = super().clean_code()
+        if code:
+            # We have a valid code. But, can we actually perform the change?
+            email_already_exists(user=self.user, email=self.email, always_raise=True)
+        return code
 
 
 class ConfirmPasswordResetCodeForm(BaseConfirmCodeForm):
@@ -852,18 +810,46 @@ class ConfirmPasswordResetCodeForm(BaseConfirmCodeForm):
 
 
 class VerifyPhoneForm(BaseConfirmCodeForm):
-    pass
+    def __init__(self, *args, **kwargs) -> None:
+        self.user = kwargs.pop("user", None)
+        self.phone = kwargs.pop("phone", None)
+        super().__init__(*args, **kwargs)
+
+    def clean_code(self) -> str:
+        code = super().clean_code()
+        if code:
+            # We have a valid code. But, can we actually perform the change?
+            phone_already_exists(self.user, self.phone, always_raise=True)
+        return code
 
 
 class ChangePhoneForm(forms.Form):
     def __init__(self, *args, **kwargs):
-        self.phone = kwargs.pop("phone")
+        self.user = kwargs.pop("user", None)
+        self.phone = kwargs.pop("phone", None)
         super().__init__(*args, **kwargs)
         adapter = get_adapter()
         self.fields["phone"] = adapter.phone_form_field(required=True)
 
     def clean_phone(self):
         phone = self.cleaned_data["phone"]
+        adapter = get_adapter()
         if phone == self.phone:
-            raise get_adapter().validation_error("same_as_current")
+            raise adapter.validation_error("same_as_current")
+        self.account_already_exists = phone_already_exists(self.user, phone)
         return phone
+
+
+class ChangeEmailForm(forms.Form):
+    email = EmailField(required=True)
+
+    def __init__(self, *args, **kwargs):
+        self.email = kwargs.pop("email", None)
+        super().__init__(*args, **kwargs)
+
+    def clean_email(self):
+        email = self.cleaned_data["email"]
+        if email == self.email:
+            raise get_adapter().validation_error("same_as_current")
+        email, self.account_already_exists = email_already_exists(email)
+        return email
