@@ -3,6 +3,7 @@ from typing import Optional
 
 from django.db import models
 from django.db.models import Q
+from django.http import HttpRequest
 from django.utils import timezone
 
 from . import app_settings
@@ -25,21 +26,19 @@ class EmailAddressManager(models.Manager):
         """
         Returns the email address the user is in the process of changing to, if any.
         """
-        assert app_settings.CHANGE_EMAIL  # nosec
-        return (
-            self.model.objects.filter(user=user, verified=False).order_by("pk").last()
-        )
+        return self.filter(user=user, verified=False).order_by("pk").last()
 
-    def add_new_email(self, request, user, email):
+    def add_new_email(
+        self, request: HttpRequest, user, email: str, send_verification: bool = True
+    ):
         """
         Adds an email address the user wishes to change to, replacing his
         current email address once confirmed.
         """
-        assert app_settings.CHANGE_EMAIL  # nosec
         instance = self.get_new(user)
         email = email.lower()
         if not instance:
-            instance = self.model.objects.create(user=user, email=email)
+            instance = self.create(user=user, email=email)
         else:
             # Apparently, the user was already in the process of changing his
             # email.  Reuse that temporary email address.
@@ -47,7 +46,8 @@ class EmailAddressManager(models.Manager):
             instance.verified = False
             instance.primary = False
             instance.save()
-        instance.send_confirmation(request)
+        if send_verification:
+            instance.send_confirmation(request)
         return instance
 
     def add_email(self, request, user, email, confirm=False, signup=False):
@@ -101,7 +101,7 @@ class EmailAddressManager(models.Manager):
         addresses = getattr(user, cache_key, None)
         email = email.lower()
         if addresses is None:
-            ret = self.get(user=user, email=email)
+            ret = self.get(user=user, email=email.lower())
             # To avoid additional lookups when e.g.
             # EmailAddress.set_as_primary() starts touching self.user
             ret.user = user
