@@ -1,7 +1,9 @@
 """Test MailChimp OAuth2 v3 Flow."""
 
+from django.test import TestCase
+
 from allauth.socialaccount.tests import OAuth2TestsMixin
-from allauth.tests import MockedResponse, TestCase
+from allauth.tests import MockedResponse
 
 from .provider import MailChimpProvider
 
